#include "ViewSynDef.h"
#include "ViewSynAPI.h"
#if POZNAN_NONLINEAR_DEPTH
#include "nonlinear_depth.h"
#endif
#include "Visbd.h"

#if defined (__cplusplus)
extern "C" 
{
#endif
#if NOKIA_DISP_CALC_B0150
	int InitViewSyn(ViewSynParams* pViewSynParams)
	{
		ViewSyn1DMode.InitViewInterpolation1D(pViewSynParams);
		return 1;
	}
#else
	int InitViewSyn(ViewSynParams* pViewSynParams,CameraParameters* pCamParams)
	{
	  ViewSyn1DMode.InitViewInterpolation1D(pViewSynParams,pCamParams);
	  return 1;
	}
#endif
#if QC_VSP_Asymmetric_Resolution
	int BeginViewSyn(ViewSynIOFrmData* pViewSynIOFrmData, Boolean bMixedRes)
	{
		int ret=0;
		ViewSyn1DMode.AllocMem(bMixedRes);
		ret=ViewSyn1DMode.DoOneFrame(pViewSynIOFrmData, bMixedRes);
		ViewSyn1DMode.FreeMem();
		return ret;
	}
#else
	int BeginViewSyn(ViewSynIOFrmData* pViewSynIOFrmData)
	{
		//ViewSyn1DMode.UpdataCamParams(pNewCamParams);
		int ret=0;
		ViewSyn1DMode.AllocMem();
		ret=ViewSyn1DMode.DoOneFrame(pViewSynIOFrmData);
		ViewSyn1DMode.FreeMem();
		return ret;
	}
#endif

#if defined (__cplusplus)
}
#endif

